<?php
/*
 * Calculate click tracks from an apache webserver log file in the combined format.
 * 
 * $Author: brucemyers $
 * $Date: 2010-07-14 22:50:02 -0400 (Wed, 14 Jul 2010) $
 * 
 * Usage: php clicktrack.php [path to logfile]
 */

	//**************************
	// Start configuration
	//**************************
	$exclude_exts = array('jpg', 'png', 'gif', 'svg', 'js', 'css', 'ico');
	$exclude_paths = array('^/kmlserver/', '^/weather_stations/display/', '^/geographics/loadMap', '^/geographics/loadGeoList',
		'^/geographics/geosfortype', '^/geographics/subdivision', '^/taxas/extref', '^/awstats', '^/robots.txt');
	$suppress_errors = true; // 400's, 500's
	$baseurls = array('http://globalspecies.org', 'http://www.globalspecies.org');
	$outfile = 'clicktrack.html';
    $stylesheet =<<<EOT
<style>
#maindiv {
	width: 100%;
}
h1 {
	text-align: center;
}
hr {
	color: #0f0;
	background-color: #0f0;
	height: 2px;
	border: none;
}
td {
	vertical-align: top;
}
.altrow {
	background: #f5f5f5;
}
</style>
EOT;
	//**************************
	// End configuration
	//**************************
	
	require_once 'robots.php'; // Taken from awstats

	if ($_SERVER['argc'] < 2) {
   		echo "Usage: php clicktrack.php [path to logfile]\n";
   		exit(1);
	}
	
	$logpath = $_SERVER['argv'][1];

   	$handle = @fopen($logpath, 'r');
	if (! $handle) {
		echo 'Logfile not found ' . $logpath . "\n";
		exit(2);
	}
	
	// Compute the base url lengths.
	$baseurllens = array();
	foreach ($baseurls as $baseurl) {
		$baseurllens[] = strlen($baseurl);
	}
	
	$oh = fopen($outfile, 'w');
	
	// Fixup the exclude paths and robots into regexs.
	foreach ($exclude_paths as $key => $path) {
		$exclude_paths[$key] = '"' . $path . '"i';
	}
	
	foreach ($RobotsSearchIDOrder_list1 as $key => $path) {
		$RobotsSearchIDOrder_list1[$key] = '"' . $path . '"i';
	}
	
	foreach ($RobotsSearchIDOrder_list2 as $key => $path) {
		$RobotsSearchIDOrder_list2[$key] = '"' . $path . '"i';
	}
	
	foreach ($RobotsSearchIDOrder_listgen as $key => $path) {
		$RobotsSearchIDOrder_listgen[$key] = '"' . $path . '"i';
	}
	
	$users = array(); // keyed by ip address and user agent
	
	// Iterate over the lines in the log file.
    while (! feof($handle)) {
        $buffer = trim(fgets($handle));
        if (empty($buffer)) continue;
        
        // 127.0.0.1 - - [14/Jul/2010:06:28:48 -0400] "GET /ntaxa/1188459 HTTP/1.1" 200 10815 "http://www.google.fi/url?sa=t&source=web&cd=8&ved=0CDoQFjAH&url=http%3A%2F%2Fglobalspecies.org%2Fntaxa%2F1188459&rct=j&q=musa%20insularimontana" "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0;.NET CLR 2.0.50727; Media Center PC 5.0;)"
        
        if (preg_match('/([^ ]+) [^ ]+ [^ ]+ \[([^\]]+)\] "[^ ]+ ([^ ]+)[^"]+" (\d+) ([^ ]+) "([^"]*)" "([^"]*)"/', $buffer, $matches) == 0) {
        	echo 'Regex not found ' . $buffer . "\n";
        	continue;
        }
        
        array_shift($matches);
        list($ip, $time, $request, $status, $length, $referrer, $agent) = $matches;
        
        if (! exclude_request($request, $agent, $status)) {
        	foreach ($baseurls as $key => $baseurl) {
        		if (substr($referrer, 0, $baseurllens[$key]) == $baseurl) {
        			$referrer = substr($referrer, $baseurllens[$key]);
        			break;
        		}
        	}
        	
        	$key = md5($ip . $agent); // Make key a reasonable length.
        	$users[$key][] = array('time' => $time, 'request' => htmlentities(str_replace('&', ' ', urldecode($request)), ENT_QUOTES, 'ISO-8859-1'), 'status' => $status,
        		'length' => $length, 'referrer' =>  htmlentities(str_replace('&', ' ', urldecode($referrer)), ENT_QUOTES, 'ISO-8859-1'));
        }
    }
    
    fputs($oh, "<html><head><title>Click Track</title>\n");
    
	fputs($oh, $stylesheet . "\n");
	fputs($oh, '<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />');
    fputs($oh, "\n</head><body><div id='maindiv'><h1>Click Track</h1><table><thead><tr><th>Request</th><th>Referrer</th><th>Time / Delay</th><th>Status</th><th>Length</th></tr></thead><tbody>\n");
    
    foreach ($users as $key => $userdata) {
    	$previous_time = '';
    	$previous_url = '';
        	
    	fputs($oh, "<tr><td colspan='5'><hr /></td></tr>");
    	$altstyle = false;
    	
    	foreach ($userdata as $data) {
    		// Compute the time difference.
    		$time = $data['time'];
    		if (! empty($previous_time)){
    			$previous_time = strtotime($previous_time);
    			$current_time = strtotime($time);
    			$diff = $current_time - $previous_time;
    			$days = floor($diff / 86400);
    			$diff -= $days * 86400;
    			$hours = floor($diff / 3600);
    			$diff -= $hours * 3600;
    			$mins = floor($diff / 60);
    			$diff -= $mins * 60;
    			$secs = $diff;
    			$time = '';
    			if ($days != 0) $time .= $days . ' ';
    			if ($hours != 0 || ! empty($time)) {
    				if (! empty($time) && $hours < 10) $time .= '0';
    				$time .= $hours . ':';
    			}
    			if ($mins != 0 || ! empty($time)) {
    				if (! empty($time) && $mins < 10) $time .= '0';
    				$time .= $mins . ':';
    			}
    		    if ($secs != 0 || ! empty($time)) {
    				if (! empty($time) && $secs < 10) $time .= '0';
    				$time .= $secs;
    			}
    		} else $time = str_replace('/', '&#8209;', substr($time, 0, -6));
    		
    		if ($altstyle) $style = " class='altrow'";
    		else $style = '';
    		$referrer = $data['referrer'];
    		if ($referrer == $previous_url) $referrer = '..';
	            
    		fputs($oh, "<tr{$style}><td><a href='{$baseurls[0]}{$data['request']}'>{$data['request']}</a></td><td>{$referrer}</td><td style='text-align: right;'>{$time}</td><td style='text-align: center;'>{$data['status']}</td><td>{$data['length']}</td></tr>\n");
    		
    		$previous_time = $data['time'];
    		$previous_url = $data['request'];
    		$altstyle = ! $altstyle;
       	}
    }
    
    fputs($oh, "</tbody></table></div></body></html>\n");
    
    fclose($handle);
    fclose($oh);
    echo 'Output is in ' . $outfile . "\n";

    /**
     * Check to see if need to exclude the request based on the url, agent or status.
     * 
     * @param $url string Requested url
     * @param $agent string Requesting agent
     * @param $status int Request status
     * @return bool Exclude it?
     */
    function exclude_request($url, $agent, $status)
    {
    	global $exclude_exts, $exclude_paths, $RobotsSearchIDOrder_list1, $RobotsSearchIDOrder_list2, $RobotsSearchIDOrder_listgen,
    		$suppress_errors;
    	
    	// Check for errors.
    	if ($suppress_errors && ($status[0] == '4' || $status[0] == '5')) return true;
    	
    	// Check the file extension.
    	if (preg_match('"\.(\w+)(?:$|\?|#)"', $url, $matches) == 1 && in_array($matches[1], $exclude_exts)) return true;
    	
    	// Check the exclude paths.
    	foreach ($exclude_paths as $expath) {
    		if (preg_match($expath, $url) == 1) return true;
    	}
    	
    	// Check for robots in the agent.
    	foreach ($RobotsSearchIDOrder_list1 as $robot) {
    		if (preg_match($robot, $agent) == 1) return true;
    	}
    	foreach ($RobotsSearchIDOrder_list2 as $robot) {
    		if (preg_match($robot, $agent) == 1) return true;
    	}
    	foreach ($RobotsSearchIDOrder_listgen as $robot) {
    		if (preg_match($robot, $agent) == 1) return true;
    	}
    	
    	return false;
    }
?>